
;________________________________________________________________________
;
;                             P E E T C R. A S M
;________________________________________________________________________
;
;    Peet Bros Complete Record WX code Copyright 1999 by Steve Dimse K4HG
;
; This software is  available only for users of  the TAPR PIC-Encoder and 
; only for  non-commercial  amateur radio  or  educational  applications. 
; ALL other uses  prohibited.  This software may be modified only  if the
; resulting code be  made public,  and the original author  given credit.
; Contact the author or  TAPR  for  information  on commercial  licenses.

;________________________________________________________________________

	cblock
		hex1
		hex2
		hex3
		directionH
		directionL
		speed
		speedH
		peak5L
		peak5H
		tempL
		tempH
		rainL
		rainH
		baroL
		baroH
		baroX
		humidL
		humidH
		dateL
		dateH
		timeL
		timeH
		offsetR1L
		offsetR1H
		rain24L
		rain24H
		rain1L
		rain1H
		gustL
		gustH
	endc

;	Header = &CR& 
;	1. Wind Speed (0.1kph) 
;	2. Current Wind Direction (0-255) 
;	3. 5 minute Wind Speed Peak (0.1 kph) 
;	4. 5 minute Wind Direction Peak (0-255) 
;	5. Wind Chill (0.1 deg F) 
;	6. Outdoor Temp (0.1deg F)(corrected 5/19/97) 
;	7. Rain Total for today (0.01 in.) 
;	8. Barometer (0.1 mbar) 
;	9. Barometer Delta Value (0.1 mbar) 
;	10. Barometer Correction Factor(LSW) 
;	11. Barometer Correction Factor(MSW) 
;	12. Indoor Temp (0.1 deg F) 
;	13. Outdoor Humidity (.1%) 
;	14. Indoor Humidity (.1%) 
;	15. Dew Point (0.1 deg F) 
;	16. Date (day of year) 
;	17. Time (minute of day) 
;   98 data fields omitted!
;	115. 1 Minute Wind Speed Average (0.1kph) 
;	Total size: 448 or 452 characters (hex digits) + header, carriage return and line feed. 
;	Carriage Return & Line Feed 


hex2bin	addlw	0 - 'A'		; convert one char to hex
	btfss	STATUS, C
	addlw	.7		; Number
	addlw	.10
	andlw	0x0f		; Safety...
	return

hexbyte	movf	hex1,w		; convert two hex chars to one byte
	call	hex2bin
	movwf	hex1
	swapf	hex1,f
	movf	hex2,w
	call	hex2bin
	addwf	hex1,w
	return

hexWord				; get 4 hex bytes and produce two bytes (in hex1 and hex2)
	call	serialReceive
	movwf	hex1
	call	serialReceive
	movwf	hex2
	call	hexbyte
	movwf	hex3
	call	serialReceive
	movwf	hex1
	call	serialReceive
	movwf	hex2
	call	hexbyte
	movwf	hex2
	movf	hex3,w
	movwf	hex1
	return

wxInit
	clrf	gustL
	clrf	gustH
	clrf	rain1H
	clrf	rain1L
	clrf	rain24H
	clrf	rain24L
	return
wxDecode			; called to extract paramters from serial stream
				;  when called, first char of string ('&') 
				;  has already been heard
	bsf	VALID_OUT	; turn on valid LED
	call	serialReceive
	xorlw	'C'		; verify U-2000 complete record string
	skpz	
	return			; not valid data
	call	serialReceive
	xorlw	'R'		; verify U-2000 complete record string
	skpz	
	return			; not valid data
	call	serialReceive
	xorlw	'&'		; verify U-2000 complete record string
	skpz	
	return			; not valid data
; Extract fields we need
; can re-write this with indirect addressing if need bytes later

	call	hexWord
	movf	hex1,w
	movwf	speedH
	movf	hex2,w
	movwf	speed

	call	hexWord
	movf	hex2,w
	movwf	directionL	; only care about low byte

	call	hexWord
	movf	hex1,w
	movwf	peak5H
	movf	hex2,w
	movwf	peak5L

	call	hexWord		; skip 5 min peak direction
	call	hexWord		; skip wind chill

	call	hexWord
	movf	hex1,w
	movwf	tempH
	movf	hex2,w
	movwf	tempL

	call	hexWord
	movf	hex1,w
	movwf	rainH
	movf	hex2,w
	movwf	rainL

	call	hexWord
	movf	hex1,w
	movwf	baroH
	movf	hex2,w
	movwf	baroL

	call	hexWord		; skip 4 bytes for baro delta
	call	hexWord		; skip 8 bytes for baro correction
	call	hexWord
	call	hexWord		; skip 4 bytes for indoor temp

	call	hexWord
	movf	hex1,w
	movwf	humidH
	movf	hex2,w
	movwf	humidL

	call	hexWord		; skip 4 bytes for indoor humidity
	call	hexWord		; skip 4 bytes for dew point

	call	hexWord
	movf	hex1,w
	movwf	dateH
	movf	hex2,w
	movwf	dateL

	call	hexWord
	movf	hex1,w
	movwf	timeH
	movf	hex2,w
	movwf	timeL


convertMPH			; wind speed in 0.1 kph, change to MPH (div by 16)
	movlw	H'F0'
	andwf	speed,f		; throw out low nybble
	movlw	H'0F'	
	andwf	speedH,w	; high nybble of speed
	addwf	speed,f
	swapf	speed,f		; nybbles are backward, fix it

	movlw	H'F0'		; do same for 5 min gust reading
	andwf	peak5L,f
	movlw	H'0F'	
	andwf	peak5H,w
	addwf	peak5L,f
	swapf	peak5L,f
	
checkGust			;check for new high wind gust
	movf	gustL,w
	subwf	peak5L,w
	skpc
	goto	notPeak
	movf	peak5L,w
	movwf	gustL
notPeak

	bcf	VALID_OUT	; turn off valid LED

;**** add check of gust field as well! ****
	return

convertWX
; Modify the data to be in BCD format before Key-up,
; so there are no conversion delays while transmitting
;  Also fix units of a few of the fields

	movlw	speed		; BCD conversion of speed
	movwf	fsr
	call	bin2pack

				; Next, fix direction...from 0...255 to BCD degrees
				; There are a couple of degrees of slop in this, but
				;  close enough for government work
	clrf	directionH	;
	decf	directionH,f
	movf	directionL,w
; ???? Do I need to special-case 255? Now it returns 360...
				; Here we fix some of the slop, needed to 
				;   prevent ff parsing as 364 degrees!
	swapf	directionL,w
	movwf	hex1
	rrf	hex1,w
	rrf	hex1,w
	andlw	B'00000011'	; Mask off original high bits
	subwf	directionL,w	;
	
dirLoop1
	incf	directionH,f
	addlw	D'186'		; (256-70) 70 counts per 100 degrees (256/360)
				;  really 71, but it is more important that it match
				;  with the 7 used below.
				;  This is why there is a need for the slop adjustment earlier
				;  and why 255 = 360 degrees
	skpnc
	goto	dirLoop1
	
	addlw	D'70'		; restore remainder
	clrf	directionL
	decf	directionL,f
dirLoop2
	incf	directionL,f
	addlw	D'249'		; (256-7) 7 counts per 10 degrees
	skpnc
	goto	dirLoop2

	addlw	D'7'
	swapf	directionL,f
	decf	directionL,f
dirLoop3
	incf	directionL,f
	addlw	D'255'
	skpc	
	goto	dirDone
	incf	directionL,f
	incf	directionL,f
	addlw	D'255'
	skpnc
	goto	dirLoop3
	decf	directionL,f
dirDone
	movlw	gustL		; BCD conversion of gust
	movwf	fsr
	call	bin2pack

				; convert time to hours/minutes
	clrf	hex1
timeLoop
	movlw	.60
	incf	hex1,f
	subwf	timeL,f
	skpnc
	goto	timeLoop
	movlw	1
	subwf	timeH,f
	skpnc
	goto	timeLoop
	decf	hex1,w		; overshot by one hour
	movwf	timeH		; store in high byte of time
	movlw	.60
	addwf	timeL,f		; restore remainder (minutes)
	movlw	timeL
	movwf	fsr
	call	bin2byte	; convert to BCD
	movlw	timeH
	movwf	fsr
	call	bin2byte	; convert to BCD

	clrf	hex1
	

	movlw	tempL
	movwf	fsr
	call	bin2pack

	movlw	baroL
	movwf	fsr
	call	bin2pack
	movf	tenk,w
	movwf	baroX		; have to save top byte for this one only

	movlw	humidL
	movwf	fsr
	call	bin2pack

convertDate			; Now we convert day of year to month and day
	clrf	hex1

dateLoop
	movf	hex1,w
	incf	hex1,f
	call	monthTab
	subwf	dateL,f
	btfsc	status,c
	goto	dateLoop
	movlw	1
	subwf	dateH,f
	btfsc	status,c
	goto	dateLoop
	decf	hex1,w
	call	monthTab
	addwf	dateL,f
	movf	hex1,w		; Save month	
	movwf	dateH
	incf	dateL,f 	; convert from Jan 1 = 0/0 to 1/1

	movlw	dateL		; convert to BCD
	movwf	fsr
	call	bin2byte
	movlw	dateH
	movwf	fsr
	call	bin2byte


	return
